local me = {
	name = "yaCardBook",

	FrameItemListSize = 18,

	CardList_Sorted = {},
	CardList_Filtered = {},
	NumOwned_Filtered = 0,
	ItemListSelected = 1,

	ciGrey50 = "|cff808080%s|r", -- text grey 50%

	DD_Zone_Value = -1,
	DD_Category_Value = -1,
	DD_Bonus_Value = -1,
	DD_FiltMisc_Value = -1,
	DD_Sort_Value = -1,

	Sort_Price_Sections = { 999999999, 5000000, 2000000, 1000000, 500000, 200000, 100000, 50000, 1 },
	
	FilterBox_Value = "",
	FilterBox_Num = 0,
	ScrollBar_Value = 0
}
--[[ =================================================================
  Retrieves a node from a hierarchic structured table. A valid node
  must be a table element named 'child'. Optimized for menu structures
================================================================= --]]
function me.GetIndexedNodeFromTab(tab, path)
	local result = tab;
	for idx in string.gmatch(path, "(%d+).?") do
		idx = tonumber(idx);
		if result[idx] and result[idx].child then
			result = result[idx].child;
		else
			return {};
		end
	end
	return result;
end

--[[ =================================================================
  Adds a menu structure to the currently opened menu (dropdown or else)
  If isTitle is true that entry is disabled and it's color is yellow
================================================================= --]]
function me.ShowMenu(ddname, menu, checkval)
	if type(ddname) ~= "string" or ddname == "" or type(menu) ~= "table" then
		return;
	end

	local func = me[ddname .. "_Click"];
	if type(func) ~= "function" then return; end

	local MenuLevel = UIDROPDOWNMENU_MENU_LEVEL;
	local SubMenuKey = "child";
	local entries = menu;
	if MenuLevel > 1 then
		SubMenuKey = UIDROPDOWNMENU_MENU_VALUE;
		entries = me.GetIndexedNodeFromTab(menu, UIDROPDOWNMENU_MENU_VALUE);
	end

	for idx, entry in pairs(entries) do
		local info = {};
		info.text = entry.text;
		info.value = iif(entry.child, SubMenuKey .. "." .. idx, entry.value);
		info.checked = (checkval == info.value);
		info.isTitle = entry.isTitle;
		info.notCheckable = entry.isTitle;
		info.disabled = entry.disabled;
		info.markNumber = entry.markNumber;
		info.notCheckable = entry.notCheckable;
		info.hasArrow = iif(entry.child, 1, nil);
		info.func = iif(entry.child, nil, func);
		info.owner = getglobal(me.name .. "_" .. ddname);
		UIDropDownMenu_AddButton(info, MenuLevel);
	end
end

--[[ =================================================================
  Searches a menu structure for an option with the given value
  and returns the related text label; otherwise nil
================================================================= --]]
function me.GetMenuTextFromValue(menu, value)
local result = nil;
	for idx, entry in pairs(menu) do
		if entry.child then 
			result = me.GetMenuTextFromValue(entry.child, value);
		else
			if entry.value == value then result = entry.text; end
		end
		if result then return result; end
	end
end

--[[ =================================================================
  Sets a dropdown menu with the given value
================================================================= --]]
function me.SetMenuValue(DDname, value)
	local menu = me[DDname .. "_Menu"];
	if not menu then return; end
	local DD_Text = getglobal(me.name .. "_" .. DDname .. "Text");
	DD_Text:SetText(me.GetMenuTextFromValue(menu, value));
	me[DDname .. "_Value"] = value;
end

--[[ =================================================================
  DropDown menu to select/filter zone (me.name .. "_DD_Zone")
  Standard definition set: 
    value variable, menu entry table, 
    open-, show- and click-event, create menu and filter function
================================================================= --]]
function me.Zone_Populated(zoneID)
	if yaCIt.ZoneLookup[zoneID] then
		for _, val in pairs(yaCIt.ZoneLookup[zoneID]) do
			if val ~= -1 then return true; end
		end
	end
	return false;
end
				
function me.DD_Zone_CreateMenu()
	if yaCIt.ZoneSelectDDneedUpdate and table.getn(yaCIt.ZoneLookup) > 0 then
		me.DD_Zone_Menu = {};
		local strLocales = yaCIt.Locales.book.zone;
		table.insert(me.DD_Zone_Menu, {text = strLocales.title, isTitle = 1, notCheckable = 1});
		table.insert(me.DD_Zone_Menu, {value = -1, text = strLocales.allcards});
		table.insert(me.DD_Zone_Menu, {value = -3, text = strLocales.current});
		table.insert(me.DD_Zone_Menu, {value = -2, text = strLocales.unknown});

		local continentID = 1; -- Only one continent
		local mapTables = WorldMapTable[continentID].mapTables;

		for mapTypeIndex, mapType in ipairs(g_MapTypes) do
			local mapTypeName = TEXT(string.upper("UI_WORLDMAP_" .. mapType));
			local MenuEntry = {text = mapTypeName, child = {}};
			local mapTypePopulated = false;

			for _, zone in ipairs(mapTables[mapType]) do
			if ( zone.id == 182 ) then
					zone.id = 181;
		    elseif ( zone.id == 185 ) then
		    		zone.id = 184;
		    elseif ( zone.id == 188 ) then
		    		zone.id = 187;
		    elseif ( zone.id == 255 ) then
		    		zone.id = 254;
			elseif ( zone.id == 122 ) then
					zone.id = 961;
			elseif ( zone.id == 971 ) then
					zone.id = 970
		    elseif ( zone.id == 252 ) then
		    		zone.id = 962;
				end
				if me.Zone_Populated(zone.id) then
					mapTypePopulated  = true;
					table.insert(MenuEntry.child, { value = zone.id, text = zone.name .. " " .. "(" .. zone.id .. ")" } );
				end
			end

			if mapTypePopulated then
				table.insert(me.DD_Zone_Menu, MenuEntry);
			end
		end
		yaCIt.ZoneSelectDDneedUpdate = false;
	end
end

function me.DD_Zone_OnLoad(this)
	UIDropDownMenu_SetWidth(this, 120);
	UIDropDownMenu_SetText(this, yaCIt.Locales.book.zone.allcards);
	UIDropDownMenu_Initialize(this, me.DD_Zone_Show);
end

function me.DD_Zone_Show()
	me.DD_Zone_CreateMenu();
	if me.DD_Zone_Menu then me.DD_Zone_Menu[4].disabled = iif(me.Zone_Populated(yaCIt.GetZoneID()), nil, 1); end
	me.ShowMenu("DD_Zone", me.DD_Zone_Menu, me.DD_Zone_Value);
end

function me.DD_Zone_Click(button)
	if me.DD_Zone_Value ~= button.value then
		if button.value == -3 then
			UIDropDownMenu_SetText(button.owner, GetZoneLocalName(yaCIt.GetZoneID()));
		else
			UIDropDownMenu_SetText(button.owner, button:GetText());
		end
		me.DD_Zone_Value = button.value;
		me.CreateFilteredList();
		me.UpdateFrame();
	end
	CloseDropDownMenus();
end

function me.DD_Zone_Filter(strucCard)
	if me.DD_Zone_Value == -1 then
		return true;
	elseif me.DD_Zone_Value == -2 then
		for _, val in pairs(strucCard.Zones) do
			if val ~= -1 then return false; end
		end
		return true;
	end
	
	local zoneID = iif(me.DD_Zone_Value == -3,  yaCIt.GetZoneID(), me.DD_Zone_Value);
	if not strucCard.Zones[zoneID] then return false; end
	if strucCard.Zones[zoneID] == -1 then return false; end

	return true;
end

--[[ =================================================================
  DropDown menu to select/filter card category (me.name .. "_DD_Category")
  Standard definition set: 
    value variable, menu entry table, 
    open-, show- and click-event, create menu and filter function
================================================================= --]]
function me.DD_Category_CreateMenu()
	if type(me.DD_Category_Menu) ~= "table" then
		me.DD_Category_Menu = {};
		local strLocales = yaCIt.Locales.book.category;
		table.insert(me.DD_Category_Menu, {text = strLocales.title, isTitle = 1, notCheckable = 1});
		table.insert(me.DD_Category_Menu, {value = -1, text = strLocales.allcards});

		for idx = 0, yaCIt.NumCategories do
			if LuaFunc_GetCardMaxCount(idx) > 0 then
				local name = LuaFunc_GetString("SYS_RACE_" .. idx);
				table.insert(me.DD_Category_Menu, {value = idx, text = name});
			end
		end
	end
end

function me.DD_Category_OnLoad(this)
	UIDropDownMenu_SetWidth(this, 120);
	UIDropDownMenu_SetText(this, yaCIt.Locales.book.category.allcards);
	UIDropDownMenu_Initialize(this, me.DD_Category_Show);
end

function me.DD_Category_Show()
	me.DD_Category_CreateMenu();
	me.ShowMenu("DD_Category", me.DD_Category_Menu, me.DD_Category_Value);
end

function me.DD_Category_Click(button)
	if me.DD_Category_Value ~= button.value then
		UIDropDownMenu_SetText(button.owner, button:GetText());
		me.DD_Category_Value = button.value;
		me.CreateFilteredList();
		me.UpdateFrame();
	end
end

function me.DD_Category_Filter(strucCard)
	return iif(not me.DD_Category_Value or me.DD_Category_Value == -1, true, strucCard.RaceID == me.DD_Category_Value);
end

--[[ =================================================================
  DropDown menu to select/filter stat bonus (me.name .. "_DD_Bonus")
  Standard definition set: 
    value variable, menu entry table, 
    open-, show- and click-event, create menu and filter function
================================================================= --]]
function me.DD_Bonus_CreateMenu()
	me.DD_Bonus_Menu = {};
	local strLocales = yaCIt.Locales.book.bonus;
	table.insert(me.DD_Bonus_Menu, {text = strLocales.title, isTitle = 1, notCheckable = 1});
	table.insert(me.DD_Bonus_Menu, {value = -1, text = strLocales.allcards});
 -- table.insert(me.DD_Bonus_Menu, {value = -2, text = strLocales.unknown});

	for bonus, data in pairs(yaCIt.StatBonuses) do
		table.insert(me.DD_Bonus_Menu, {value = bonus, text = bonus});
	end
end

function me.DD_Bonus_OnLoad(this)
	UIDropDownMenu_SetWidth(this, 120);
	UIDropDownMenu_SetText(this, yaCIt.Locales.book.bonus.allcards);
	UIDropDownMenu_Initialize(this, me.DD_Bonus_Show);
end

function me.DD_Bonus_Show()
	me.DD_Bonus_CreateMenu();
	me.ShowMenu("DD_Bonus", me.DD_Bonus_Menu, me.DD_Bonus_Value);
end

function me.DD_Bonus_Click(button)
	if me.DD_Bonus_Value ~= button.value then
		UIDropDownMenu_SetText(button.owner, button:GetText());
		me.DD_Bonus_Value = button.value;
		me.CreateFilteredList();
		me.UpdateFrame();
	end
end

function me.DD_Bonus_Filter(strucCard)
	if not me.DD_Bonus_Value or me.DD_Bonus_Value == -1 then
		return true;
	elseif me.DD_Bonus_Value == -2 then
		return strucCard.Stat:len() == 0;
	else
	 -- return strucCard.Stat:sub(4) == me.DD_Bonus_Value;
		return strucCard.Stat:gsub( ".%d+ ?", "" ) == me.DD_Bonus_Value; -- if percent dmg is same value for both, so dropdown is showing wrong fixed +1% for mdmg - remove `^` symbol from begin
	end
end

--[[ =================================================================
  DropDown menu to select/filter misc. options (me.name .. "_DD_FiltMisc")
  Standard definition set: 
    value variable, menu entry table, 
    open-, show- and click-event, create menu and filter function
================================================================= --]]
function me.DD_FiltMisc_CreateMenu()
	me.DD_FiltMisc_Menu = {};
	local strLocales = yaCIt.Locales.book.misc;
	table.insert(me.DD_FiltMisc_Menu, {text = strLocales.title, isTitle = 1, notCheckable = 1});
	table.insert(me.DD_FiltMisc_Menu, {value = -1, text = strLocales.allcards});
	
	if yaCIt_Persistent then
		local MenuEntry = {text = strLocales.owned, child = {}};
		for CharName, Data in pairs(yaCIt_Persistent[yaCIt.Realm]) do
			
			if Data.Owned then
				table.insert(MenuEntry.child, {value = CharName, text = CharName, markNumber = iif(CharName == yaCIt.Player, 1, nil)});
			end
		end
		table.insert(me.DD_FiltMisc_Menu, MenuEntry);
	end

	--- Event Cards -------------------
	local MenuEntryEvent = {text = strLocales.event, child = {}};
    table.insert(MenuEntryEvent.child, {value = -9, text = strLocales.fireevent}); -- Event Fire Festival --
    table.insert(MenuEntryEvent.child, {value = -10, text = strLocales.flowerevent}); -- Event Flower Festival --
    table.insert(MenuEntryEvent.child, {value = -11, text = strLocales.maskedball}); -- Event Masked Ball Festival --
    table.insert(MenuEntryEvent.child, {value = -12, text = strLocales.springrainevent}); -- Event Spring Rain Festival --
    table.insert(MenuEntryEvent.child, {value = -13, text = strLocales.musicevent}); -- Event Music Festival --
    table.insert(MenuEntryEvent.child, {value = -14, text = strLocales.craftingevent}); -- Event Crafting Festival --
    table.insert(MenuEntryEvent.child, {value = -15, text = strLocales.colorweaveevent}); -- Event Color Weave Festival --
    table.insert(MenuEntryEvent.child, {value = -16, text = strLocales.fairytailevent}); -- Event Fairy Tail Festival --
 -- table.insert(MenuEntryEvent.child, {value = -17, text = strLocales.anniversaryevent}); -- Event Anniversary Festival --
    table.insert(MenuEntryEvent.child, {value = -18, text = strLocales.juiceevent}); -- Event Juice Festival --
    table.insert(MenuEntryEvent.child, {value = -19, text = strLocales.pumpkinevent}); -- Event Pumpkin Festival --
    table.insert(MenuEntryEvent.child, {value = -20, text = strLocales.autumnbanquetevent}); -- Event Autumn Banquet Festival --
    table.insert(MenuEntryEvent.child, {value = -21, text = strLocales.snowflakeevent}); -- Event Snowflake Festival --
	table.insert(me.DD_FiltMisc_Menu, MenuEntryEvent);

	--- Dracos Cards -------------
	local MenuEntryEvent = {text = strLocales.draco, child = {}};
	table.insert(MenuEntryEvent.child, {value = -22, text = strLocales.dracomonster}); -- Draco Cards --
	table.insert(MenuEntryEvent.child, {value = -23, text = strLocales.dracomonstersecret}); -- Draco Cards Special --
	table.insert(me.DD_FiltMisc_Menu, MenuEntryEvent);
	
	--- Item related Cards -------------
	local MenuEntryEvent = {text = strLocales.monster, child = {}};
    table.insert(MenuEntryEvent.child, {value = -26, text = strLocales.monsteritem}); -- Item Related Monster --
    table.insert(MenuEntryEvent.child, {value = -27, text = strLocales.monsteritemspecial}); -- Item Related Monster Special --
	table.insert(me.DD_FiltMisc_Menu, MenuEntryEvent);
	
	--- Quest related Cards -------------
	local MenuEntryEvent = {text = strLocales.quest, child = {}};
    table.insert(MenuEntryEvent.child, {value = -4, text = strLocales.questmonsterI}); -- Quest Related Monster I --
	table.insert(MenuEntryEvent.child, {value = -5, text = strLocales.questmonsterII}); -- Quest Related Monster II --
	table.insert(MenuEntryEvent.child, {value = -6, text = strLocales.questmonsterspecial}); -- Quest Related Monster Special --
	table.insert(me.DD_FiltMisc_Menu, MenuEntryEvent);

	--- Quest reward Cards -------------
	local MenuEntryEvent = {text = strLocales.reward, child = {}};
	table.insert(MenuEntryEvent.child, {value = -2, text = strLocales.questreward}); -- Quest Reward --
	table.insert(MenuEntryEvent.child, {value = -3, text = strLocales.questrewardsecret}); -- Quest Reward Secret --
	table.insert(me.DD_FiltMisc_Menu, MenuEntryEvent);

	--- Monster Spawn Cards -------------
	local MenuEntryEvent = {text = strLocales.spawn, child = {}};
	table.insert(MenuEntryEvent.child, {value = -24, text = strLocales.monsterspawn}); -- Monster Spawn --
	table.insert(MenuEntryEvent.child, {value = -25, text = strLocales.monsterspawnrandom}); -- Monster Spawn Random --
	table.insert(me.DD_FiltMisc_Menu, MenuEntryEvent);

    --- Event Monster Cards -----------------	
    table.insert(me.DD_FiltMisc_Menu, {value = -8, text = strLocales.eventmonster}); -- Event Related Monster Cards --

    --- Instance Event Cards -----------------
	local MenuEntryEvent = {text = strLocales.eventinstance, child = {}};
	table.insert(MenuEntryEvent.child, {value = -35, text = strLocales.eventinstanceanniversary}); -- Event Anniversary --
	table.insert(MenuEntryEvent.child, {value = -36, text = strLocales.eventinstancespring}); -- Event Spring --
 -- table.insert(MenuEntryEvent.child, {value = -37, text = strLocales.eventinstancevalentine}); -- Event Valentine --
	table.insert(MenuEntryEvent.child, {value = -38, text = strLocales.eventinstancehalloween}); -- Event Halloween --
    table.insert(MenuEntryEvent.child, {value = -39, text = strLocales.eventinstancechristmas}); -- Event Christmas --
	table.insert(me.DD_FiltMisc_Menu, MenuEntryEvent);

    --- Package Cards -------------------
	local MenuEntryEvent = {text = strLocales.packagecard, child = {}};
	table.insert(MenuEntryEvent.child, {value = -28, text = strLocales.packagecard1}); -- Chronicler Verses Contest --
	table.insert(me.DD_FiltMisc_Menu, MenuEntryEvent);

    --- Gasha Cards -------------------
	local MenuEntryEvent = {text = strLocales.gashacard, child = {}};
	table.insert(MenuEntryEvent.child, {value = -29, text = strLocales.gashacard1}); -- Wabbit Gasha --
	table.insert(MenuEntryEvent.child, {value = -30, text = strLocales.gashacard2}); -- Edwards Family Gasha --
	table.insert(MenuEntryEvent.child, {value = -31, text = strLocales.gashacard3}); -- Christmas Gasha --
	table.insert(MenuEntryEvent.child, {value = -32, text = strLocales.gashacard4}); -- Sunken Monster Gasha --
	table.insert(me.DD_FiltMisc_Menu, MenuEntryEvent);

    --- PlayerNet Games Cards -------------------
	table.insert(me.DD_FiltMisc_Menu, {value = -33, text = strLocales.playernetgames}); -- PlayerNet Games --

    --- Future Zone Cards -------------------
	table.insert(me.DD_FiltMisc_Menu, {value = -34, text = strLocales.futurezone}); -- Future Zone --

    --- NOT IN USE AT THE MOMENT -------------------
	-- table.insert(me.DD_FiltMisc_Menu, {value = -7, text = strLocales.notobtain});

	-- if AAH_SavedHistoryTable then
	-- table.insert(me.DD_FiltMisc_Menu, {value = -28, text = strLocales.AAHhistory});
	-- end
end

function me.DD_FiltMisc_OnLoad(this)
	UIDropDownMenu_SetWidth(this, 120);
	UIDropDownMenu_SetText(this, yaCIt.Locales.book.misc.allcards);
	UIDropDownMenu_Initialize(this, me.DD_FiltMisc_Show);
end

function me.DD_FiltMisc_Show()
	me.DD_FiltMisc_CreateMenu();
	me.ShowMenu("DD_FiltMisc", me.DD_FiltMisc_Menu, me.DD_FiltMisc_Value);
end

function me.DD_FiltMisc_Click(button)
	if me.DD_FiltMisc_Value ~= button.value then
		UIDropDownMenu_SetText(button.owner, button:GetText());
		me.DD_FiltMisc_Value = button.value;
		me.CreateFilteredList();
		me.UpdateFrame();
	end
	CloseDropDownMenus();
end

function me.DD_FiltMisc_Filter(strIDhex)
	local strucCard = yaCIt.CardList[strIDhex];
	if not me.DD_FiltMisc_Value or me.DD_FiltMisc_Value == -1 then
		return true;
		
		
	elseif me.DD_FiltMisc_Value == -2 then
		return (strucCard.QuestReward == 1);
	elseif me.DD_FiltMisc_Value == -3 then
		return (strucCard.QuestRewardSecret== 1);
	elseif me.DD_FiltMisc_Value == -4 then
		return (strucCard.QuestMonsterI == 1);
	elseif me.DD_FiltMisc_Value == -5 then
		return (strucCard.QuestMonsterII == 1);
	elseif me.DD_FiltMisc_Value == -6 then
		return (strucCard.QuestMonsterSpecial == 1);
	elseif me.DD_FiltMisc_Value == -7 then
		return (strucCard.NotObtainable == 1);
	elseif me.DD_FiltMisc_Value == -8 then
		return (strucCard.EventMonster == 1);
    elseif me.DD_FiltMisc_Value == -9 then
        return (strucCard.FireEvent == 1);
    elseif me.DD_FiltMisc_Value == -10 then
        return (strucCard.FlowerEvent == 1);
    elseif me.DD_FiltMisc_Value == -11 then
        return (strucCard.MaskedBall == 1);
	elseif me.DD_FiltMisc_Value == -12 then
		return (strucCard.SpringRainEvent == 1);
	elseif me.DD_FiltMisc_Value == -13 then
		return (strucCard.MusicEvent == 1);	
	elseif me.DD_FiltMisc_Value == -14 then
		return (strucCard.CraftingEvent == 1);
	elseif me.DD_FiltMisc_Value == -15 then
		return (strucCard.ColorWeaveEvent == 1);
	elseif me.DD_FiltMisc_Value == -16 then
		return (strucCard.FairyTailEvent == 1);
 -- elseif me.DD_FiltMisc_Value == -17 then
 --     return (strucCard.AnniversaryEvent == 1);
	elseif me.DD_FiltMisc_Value == -18 then
		return (strucCard.JuiceEvent == 1);
    elseif me.DD_FiltMisc_Value == -19 then
        return (strucCard.PumpkinEvent == 1);
    elseif me.DD_FiltMisc_Value == -20 then
        return (strucCard.AutumnBanquetEvent == 1);
    elseif me.DD_FiltMisc_Value == -21 then
        return (strucCard.SnowflakeEvent == 1);
	elseif me.DD_FiltMisc_Value == -22 then
		return (strucCard.DracoMonster == 1);
	elseif me.DD_FiltMisc_Value == -23 then
		return (strucCard.DracoMonsterSecret == 1);
	elseif me.DD_FiltMisc_Value == -24 then
		return (strucCard.MonsterSpawn == 1);
	elseif me.DD_FiltMisc_Value == -25 then
		return (strucCard.MonsterSpawnRandom == 1);
	elseif me.DD_FiltMisc_Value == -26 then
		return (strucCard.MonsterItem == 1);
	elseif me.DD_FiltMisc_Value == -27 then
		return (strucCard.MonsterItemSpecial == 1);
	elseif me.DD_FiltMisc_Value == -28 then
		return (strucCard.PackageCard1 == 1);
	elseif me.DD_FiltMisc_Value == -29 then
		return (strucCard.GashaCard1 == 1);
	elseif me.DD_FiltMisc_Value == -30 then
		return (strucCard.GashaCard2 == 1);
	elseif me.DD_FiltMisc_Value == -31 then
		return (strucCard.GashaCard3 == 1);
	elseif me.DD_FiltMisc_Value == -32 then
		return (strucCard.GashaCard4 == 1);
	elseif me.DD_FiltMisc_Value == -33 then
		return (strucCard.PlayerNetGames == 1);
	elseif me.DD_FiltMisc_Value == -34 then
		return (strucCard.FutureZone == 1);
	elseif me.DD_FiltMisc_Value == -35 then
		return (strucCard.EventInstanceAnniversary == 1);
	elseif me.DD_FiltMisc_Value == -36 then
		return (strucCard.EventInstanceSpring == 1);
 -- elseif me.DD_FiltMisc_Value == -37 then
 --     return (strucCard.EventInstanceValentine == 1);
	elseif me.DD_FiltMisc_Value == -38 then
		return (strucCard.EventInstanceHalloween == 1);
    elseif me.DD_FiltMisc_Value == -39 then
        return (strucCard.EventInstanceChristmas == 1);
    ----------------------------------------

	elseif me.DD_FiltMisc_Value == -40 then
		return (AAH_SavedHistoryTable[tonumber(strIDhex,40)] ~= nil);
	end
	return yaCIt_Persistent[yaCIt.Realm][me.DD_FiltMisc_Value].Owned[strIDhex] == 1;
end

--[[ =================================================================
  DropDown menu to select primary sort order (me.name .. "_DD_Sort")
  Standard definition set: 
    value variable, menu entry table, 
    open-, show- and click-event, create menu function
================================================================= --]]
function me.DD_Sort_CreateMenu()
	me.DD_Sort_Menu = {};
	local strLocales = yaCIt.Locales.book.sort;
	table.insert(me.DD_Sort_Menu, {text = strLocales.title, isTitle = 1, notCheckable = 1});
	table.insert(me.DD_Sort_Menu, {value = -1, text = strLocales.nosort});
	table.insert(me.DD_Sort_Menu, {value = 1, text = strLocales.zone});
	table.insert(me.DD_Sort_Menu, {value = 2, text = strLocales.stat});
	table.insert(me.DD_Sort_Menu, {value = 3, text = strLocales.owned});
 -- if AAH_SavedHistoryTable then
 -- table.insert(me.DD_Sort_Menu, {value = 4, text = strLocales.AAHavg});
 -- end
end

function me.DD_Sort_OnLoad(this)
	UIDropDownMenu_SetWidth(this, 120);
	UIDropDownMenu_SetText(this, yaCIt.Locales.book.sort.nosort);
	UIDropDownMenu_Initialize(this, me.DD_Sort_Show);
end

function me.DD_Sort_Show()
	me.DD_Sort_CreateMenu();
	me.ShowMenu("DD_Sort", me.DD_Sort_Menu, me.DD_Sort_Value);
end

local AccentTable = {
	["à"] = "a",
	["á"] = "a",
	["â"] = "a",
	["ã"] = "a",
	["ä"] = "a",
	["ą"] = "a",
	["ä"] = "a",
	["ç"] = "c",
	["ć"] = "c",
	["è"] = "e",
	["é"] = "e",
	["ê"] = "e",
	["ë"] = "e",
	["ę"] = "e",
	["ì"] = "i",
	["í"] = "i",
	["î"] = "i",
	["ï"] = "i",
	["ł"] = "l",
	["ñ"] = "n",
	["ń"] = "n",
	["ò"] = "o",
	["ó"] = "o",
	["ô"] = "o",
	["õ"] = "o",
	["ö"] = "o",
	["ó"] = "o",
	["ó"] = "o",
	["ś"] = "s",
	["ß"] = "s",
	["ù"] = "u",
	["ú"] = "u",
	["û"] = "u",
	["ü"] = "u",
	["ü"] = "u",
	["ý"] = "y",
	["ÿ"] = "y",
	["ż"] = "z",
	["ź"] = "z",
	["À"] = "A",
	["Á"] = "A",
	["Â"] = "A",
	["Ã"] = "A",
	["Ä"] = "A",
	["Ą"] = "A",
	["Ä"] = "A",
	["Ç"] = "C",
	["Ć"] = "C",
	["È"] = "E",
	["É"] = "E",
	["Ê"] = "E",
	["Ë"] = "E",
	["Ę"] = "E",
	["Ì"] = "I",
	["Í"] = "I",
	["Î"] = "I",
	["Ï"] = "I",
	["Ł"] = "L",
	["Ñ"] = "N",
	["Ń"] = "N",
	["Ò"] = "O",
	["Ó"] = "O",
	["Ô"] = "O",
	["Õ"] = "O",
	["Ö"] = "O",
	["Ó"] = "O",
	["Ś"] = "S",
	["Ù"] = "U",
	["Ú"] = "U",
	["Û"] = "U",
	["Ü"] = "U",
	["Ü"] = "U",
	["Ý"] = "Y",
	["Ż"] = "Z",
	["Ź"] = "Z",
};
	
function stripAccents( str )
	if not str then return "" end
	local normalizedString = "";

	for strChar in string.gfind( str,  "([%z\1-\127\194-\244][\128-\191]*)" ) do
		if ( AccentTable[strChar] ~= nil ) then
			normalizedString = normalizedString .. AccentTable[strChar];
		else
			normalizedString = normalizedString .. strChar;
		end
	end

	normalizedString = string.gsub( normalizedString, "%[", "" );
	normalizedString = string.gsub( normalizedString, "%]", "" );

	return normalizedString;
end

function me.DD_Sort_Sort()
	me.CardList_Sorted = {};
	if me.DD_Sort_Value == -1 then
		table.sort(me.CardList_Filtered, function (c1, c2)
			return stripAccents( yaCIt.CardList[c1].cName ) < stripAccents( yaCIt.CardList[c2].cName ); 
		end)
		me.CardList_Sorted = me.CardList_Filtered;
	elseif me.DD_Sort_Value == 1 then
		table.sort(me.CardList_Filtered, function (c1, c2)
			local Zones1 = yaCIt.GetZonesString(yaCIt.CardList[c1]);
			local Zones2 = yaCIt.GetZonesString(yaCIt.CardList[c2]);
			if Zones1 == Zones2 then
				return stripAccents( yaCIt.CardList[c1].cName ) < stripAccents( yaCIt.CardList[c2].cName ); 
			else
				return Zones1 < Zones2;
			end
		end)
		local lastVal = "";
		for idx,strIDhex in ipairs(me.CardList_Filtered) do
			local zone = yaCIt.GetZonesString(yaCIt.CardList[strIDhex]);
			if zone ~= lastVal then
				lastVal = zone;
				table.insert(me.CardList_Sorted, zone);
			end
			table.insert(me.CardList_Sorted, strIDhex);
		end
	elseif me.DD_Sort_Value == 2 then
		table.sort(me.CardList_Filtered, function (c1, c2)
			local Card1 = yaCIt.CardList[c1];
			local Card2 = yaCIt.CardList[c2];
			if Card1.Stat == Card2.Stat then
				return stripAccents( Card1.cName ) < stripAccents( Card2.cName ); 
			else
				return Card1.Stat < Card2.Stat;
			end
		end)
		local lastVal = "";
		for idx,strIDhex in ipairs(me.CardList_Filtered) do
			local stat = yaCIt.CardList[strIDhex].Stat;
			if stat ~= lastVal then
				lastVal = stat;
				table.insert(me.CardList_Sorted, stat);
			end
			table.insert(me.CardList_Sorted, strIDhex);
		end
	elseif me.DD_Sort_Value == 3 then
		table.sort(me.CardList_Filtered, function (c1, c2)
			if yaCIt.Owned[c1] == yaCIt.Owned[c2] then
				return stripAccents( yaCIt.CardList[c1].cName ) < stripAccents( yaCIt.CardList[c2].cName ); 
			else
				return yaCIt.Owned[c1] > yaCIt.Owned[c2];
			end
		end)
		me.CardList_Sorted = me.CardList_Filtered;
	elseif me.DD_Sort_Value == 4 then
		table.sort(me.CardList_Filtered, function (c1, c2)
			local avg1;
			local avg2;
		  if AAH_SavedHistoryTable[tonumber(c1,16)] then
		  	avg1 = AAH_SavedHistoryTable[tonumber(c1,16)].avg;
		  else
		  	avg1 = nil;
			end
		  if AAH_SavedHistoryTable[tonumber(c2,16)] then
		  	avg2 = AAH_SavedHistoryTable[tonumber(c2,16)].avg;
		  else
		  	avg2 = nil;
			end
			if avg1 == avg2 then
				return stripAccents( yaCIt.CardList[c1].cName ) < stripAccents( yaCIt.CardList[c2].cName ); 
			else
				if not avg1 then return false; end
				if not avg2 then return true; end
				return avg1 > avg2;
			end
		end)
		local idxValue = 1;
		local MinValue = me.Sort_Price_Sections[idxValue];
		local avg;
		for idx,strIDhex in ipairs(me.CardList_Filtered) do
			if not AAH_SavedHistoryTable[tonumber(strIDhex,16)] then
				if MinValue ~= 0 then 
					table.insert(me.CardList_Sorted, yaCIt.Locales.book.sort.sectnohist);
					MinValue = 0
				end
			else
				avg = AAH_SavedHistoryTable[tonumber(strIDhex,16)].avg;
				if avg < MinValue then
					while avg < me.Sort_Price_Sections[idxValue] do 
						idxValue = idxValue + 1; 
					end
					MinValue = me.Sort_Price_Sections[idxValue];
					table.insert(me.CardList_Sorted, yaCIt.Locales.book.sort.sectavglimit .. MinValue);
				end
			end
			table.insert(me.CardList_Sorted, strIDhex);
		end
	end
end

function me.DD_Sort_Click(button)
	if me.DD_Sort_Value ~= button.value then
		UIDropDownMenu_SetText(button.owner, button:GetText());
		me.DD_Sort_Value = button.value;
		me.DD_Sort_Sort();
		me.UpdateFrame();
	end
end

--[[ =================================================================
  Card context menu to select card related action (me.name .. "_DD_CardContext")
  Standard definition set: 
    menu entry table, open-, show- and click-event, create menu function
================================================================= --]]
function me.DD_ListContext_CreateMenu()
	local strIDhex = me.CardList_Sorted[me.ItemListSelected];
	local strucCard = yaCIt.CardList[strIDhex];

	me.DD_ListContext_Menu = {};
	local strLocales = yaCIt.Locales.book.context;
	table.insert(me.DD_ListContext_Menu, {text = strLocales.title, isTitle = 1, notCheckable = 1});
	table.insert(me.DD_ListContext_Menu, {value = -1, text = CARDBOOK_TAKEOUTCARD, notCheckable = 1, disabled = iif(yaCIt.Owned[strIDhex] == 1, nil, 1)});
	table.insert(me.DD_ListContext_Menu, {value = -2, text = strLocales.linkcard, notCheckable = 1});
	local zones = yaCIt.GetZonesFromName(strucCard.cName);
	for npcZone, npcID in pairs(zones) do
		if strucCard.Zones[npcZone] ~= -1 then
			table.insert(me.DD_ListContext_Menu, {value = npcID, text = strLocales.showmap .. GetZoneLocalName(npcZone), notCheckable = 1});
		end
	end
	table.insert(me.DD_ListContext_Menu, {value = -6, text = C_CANCEL, notCheckable = 1});
end

function me.DD_ListContext_OnLoad(this)
	UIDropDownMenu_Initialize(this, me.DD_ListContext_Show, "MENU");
end

function me.DD_ListContext_Show()
	me.DD_ListContext_CreateMenu();
	me.ShowMenu("DD_ListContext", me.DD_ListContext_Menu, -99);
end

function me.DD_ListContext_Click(button)
	local action = button.value;
	local strIDhex = me.CardList_Sorted[me.ItemListSelected];
	local strucCard = yaCIt.CardList[strIDhex];
	
	if action == -1 then
		LuaFunc_TakeOutCard(tonumber(strIDhex, 16));
	elseif action == -2 then			
		local strColor = yaCIt.RarityColorsHex[strucCard.Rarity];		
		local item = yaCIt.CreateLink("item", strIDhex, yaCIt.GetCardLabel(strucCard, true), strColor);
		if not ChatEdit_AddItemLink(item) and not DEFAULT_CHAT_EDITBOX:IsVisible() then
			DEFAULT_CHAT_FRAME:AddMessage(item)
		end
	elseif action > 0 then
		ShowUIPanel(NpcTrackFrame);
		NpcTrackFrame_InitializeNpcList(action);
		NpcTrackFrame:ResetFrameOrder();
	end
end

--[[ =================================================================
  OnClick event for filtered list counts 
  Stores the current filtered list in SaveVariables.lua where they
  can be extracted after the next save
================================================================= --]]
function me.FilterCountText_OnClick(this, key)
	if IsCtrlKeyDown() and not IsShiftKeyDown() then
		_G[me.name .. "_FilterList"] = {};
		for idx, strIDhex in ipairs(me.CardList_Sorted) do
			table.insert(_G[me.name .. "_FilterList"], yaCIt.CardList[strIDhex]);
		end
		SaveVariables(me.name .. "_FilterList");
		yaCIt.Print(yaCIt.Locales.debug.savedfilterlist);
	end
end

--[[ =================================================================
  EditBox to select/filter by part of cardname (me.name .. "_FilterBox")
  A '#' followed by a 6-digit number searches for the card with ID
  Standard definition set: 
    value variable, update-event and filter function
================================================================= --]]
function me.FilterBox_Update(this)
	me.FilterBox_Num = 0;
	me.FilterBox_Value = this:GetText():lower();
	if  me.FilterBox_Value:len() == 6 and tonumber(me.FilterBox_Value) then
		me.FilterBox_Num = tonumber(me.FilterBox_Value);		
	end
	if  me.FilterBox_Value:len() == 5 and  tonumber(me.FilterBox_Value, 16) then
		me.FilterBox_Num = tonumber(me.FilterBox_Value, 16);		
	end
	me.CreateFilteredList();
	me.UpdateFrame();
end

function me.FilterBox_Filter(strIDhex)
local strLookupID;
	if me.FilterBox_Value == "" or me.FilterBox_Num == -1 then return true; end
	if me.FilterBox_Num > 0 then strLookupID = yaCIt.dec2hex(me.FilterBox_Num); end
	return strLookupID == strIDhex or yaCIt.CardList[strIDhex].cName:lower():find(me.FilterBox_Value);
end

--[[ =================================================================
  When this checkbox is clicked, just recreate filtered list and show
  Status of the checkbox is used in 'CreateFilteredList' function
================================================================= --]]
function me.Chk_Invert_Click(this)
	me.CreateFilteredList();
	me.UpdateFrame();
end

--[[ =================================================================
  Set all filters to their default value (no filter)
  While reseting the FilterBox implies the recalculation this has 
  to be called if only dropdown menus are reset
================================================================= --]]
function me.ResetFilters()
	me.SetMenuValue("DD_Zone", -1);
	me.SetMenuValue("DD_Category", -1);
	me.SetMenuValue("DD_Bonus", -1);
	me.SetMenuValue("DD_FiltMisc", -1);
	me.SetMenuValue("DD_Sort", -1);
	getglobal(me.name .. "_Chk_Invert"):SetChecked(false);

	if me.FilterBox_Value == "" then
		me.CreateFilteredList();
		me.UpdateFrame();
	else
		getglobal(me.name .. "_FilterBox"):SetText("");
	end
end

--[[ =================================================================
  Creates the filtered cardlist (indexes only)
================================================================= --]]
function me.CreateFilteredList()
	local miscstate = false;
	me.CardList_Filtered = {}
	me.NumOwned_Filtered = 0;

	for strIDhex, strucCard in pairs(yaCIt.CardList) do
		miscstate = me.DD_FiltMisc_Filter(strIDhex);
		if getglobal(me.name .. "_Chk_Invert"):IsChecked() then miscstate = not miscstate; end
		if me.DD_Zone_Filter(strucCard) and me.DD_Category_Filter(strucCard) and
			me.DD_Bonus_Filter(strucCard) and me.FilterBox_Filter(strIDhex) and miscstate then

			table.insert(me.CardList_Filtered, strIDhex);
			if yaCIt.Owned[strIDhex] == 1 then
				me.NumOwned_Filtered = me.NumOwned_Filtered + 1;
			end
		end
	end
	me.DD_Sort_Sort();
end

--[[ =================================================================
  This functions fills the 'listbox' regarding current scrollbar value
================================================================= --]]
function me.FillListbox()
	for i = 1, me.FrameItemListSize do
		local ListItem = getglobal("CardBookFrameList" .. i);
		local ListItemHL = getglobal("CardBookFrameList" .. i .. "_Select");
		local ListItemText = getglobal("CardBookFrameList" .. i .. "Text");
		local strIDhex = me.CardList_Sorted[i + me.ScrollBar_Value];
		local strucCard = yaCIt.CardList[strIDhex];

		ListItemHL:Hide();
		if i <= #me.CardList_Sorted then
			ListItem:Show();
			if strucCard then
				local strColor = iif(yaCIt.Owned[strIDhex] == 1, yaCIt.RarityColors[strucCard.Rarity], me.ciGrey50);
				ListItemText:SetText("   " .. strColor:format(strucCard.cName));
			else
				ListItemText:SetText(yaCIt.Yellow:format(me.CardList_Sorted[i + me.ScrollBar_Value]));
			end
		else
			ListItem:Hide();
		end

		if i + me.ScrollBar_Value == me.ItemListSelected then
			ListItemHL:Show();
		end
	end
end

--[[ =================================================================
  Event function when the scrollbar is moved
  --> overloaded function <--
================================================================= --]]
--function me.ScrollBar_OnValueChanged(arg1)
function CardFarmeItemListReset()
	me.ScrollBar_Value = CardBookFrameScrollBar:GetValue();
	me.FillListbox();
end

--[[ =================================================================
  Event function when the mouse wheel is used over a list item
  Event moves the scrollbar only one tick regardless of the delta value
  --> hooked function <--
  ================================================================= --]]
me.ListItem_OnMouseWheel = OnMouseWheel_CardFarmeScorllItem;
function OnMouseWheel_CardFarmeScorllItem(this, delta)
	if not CardBookFrameScrollBar:IsVisible() then
		return;
	end

	me.ListItem_OnMouseWheel(this, delta);
end

--[[ =================================================================
  Event function when  a list item is clicked
  --> overloaded function <--
================================================================= --]]
--function me.ListItem_OnClick(this, id , key)
function OnClick_CardFarmeSelectItem(this, id, key)
	CloseDropDownMenus();
	for i = 1 , me.FrameItemListSize do
		getglobal("CardBookFrameList" .. i .. "_Select"):Hide();
	end

	me.ItemListSelected = id + me.ScrollBar_Value;
	getglobal(this:GetName() .. "_Select"):Show();
	me.ShowSelectedCard();

	if key == "LBUTTON" and IsShiftKeyDown() and me.CardList_Sorted[me.ItemListSelected]:len() == 5 then
		local strIDhex = me.CardList_Sorted[me.ItemListSelected];
		local strucCard = yaCIt.CardList[strIDhex];
		local strColor = yaCIt.RarityColorsHex[strucCard.Rarity];		
		local link = yaCIt.CreateLink("item", strIDhex, yaCIt.GetCardLabel(strucCard, true), strColor);
		ChatEdit_AddItemLink( link );
	elseif key == "RBUTTON" and me.CardList_Sorted[me.ItemListSelected]:len() == 5 then
		ToggleDropDownMenu(getglobal(me.name .. "_DD_ListContext"), 1, nil, this:GetName());
	end
end

--[[ =================================================================
  Event function click on summary button
  Shows cumulated stat bonuses in InfoOverlay
================================================================= --]]
function me.ToggleSummaryButton_OnClick(this)
	if getglobal(me.name .. "_InfoOverlay"):GetID() == 200 then
		getglobal(me.name .. "_InfoOverlay"):SetID(0); -- Fallback if details hidden
		me.ShowSelectedCard();
	else
		local data = {};
		local valuepattern = yaCIt.Green .. " / " .. yaCIt.Red;

		data.ID = 200;
		for statname, totals in pairs(yaCIt.StatBonuses) do
			table.insert(data, { Left = statname .. ":", Right = valuepattern:format(totals.Own, totals.Total) } );
		end
		data.title = yaCIt.Locales.book.statoverlay.title;
		me.ShowInfoOverlay(data);
	end
end

--[[ =================================================================
  Event function click on checkbox addit. info
  Status is handled in ShowSelectedCard
================================================================= --]]
function me.Chk_ShowDetails_Click(Object)
	me.ShowSelectedCard();
end

--[[ =================================================================
  Sets anything around the card display (right part of the frame)
  for the seleceted card
================================================================= --]]
function me.ShowSelectedCard()

	me.ShowInfoOverlay(nil);

	local strIDhex = me.CardList_Sorted[me.ItemListSelected];
	local strucCard = yaCIt.CardList[strIDhex];
	getglobal(me.name .. "_CardReward"):SetText(nil);
	if not strucCard then
		getglobal(me.name .. "_CardStatus"):SetText(nil);
		CardBookFrameCard:Hide();
		return; 
	end

	CardBookFrameCard2DImage:SetTexture("interface\\CardFrame\\CardBk" .. iif(yaCIt.Owned[strIDhex] == 1, strucCard.Resist, 0));
	CardBookFrameCardName:SetText(yaCIt.RarityColors[strucCard.Rarity]:format(strucCard.cName));
	CardBookFrameCardType:SetText(LuaFunc_GetString("SYS_RACE_" .. strucCard.RaceID));	
	CardBookFrameCardType:ClearAllAnchors();
	CardBookFrameCardType:SetAnchor("TOPRIGHT","TOPRIGHT",CardBookFrame,-59,147);
	CardBookFrameCardRare:SetText(strucCard.Stat);
	getglobal(me.name .. "_MobLocation"):SetText(yaCIt.Locales.location .. yaCIt.GetZonesString(strucCard));

	local CardDescrScrollText = getglobal(me.name .. "_CardDescription");
	CardDescrScrollText:SetText(strucCard.Description);

	local CardDescrScrollbar = getglobal(me.name .. "_CardDescrScrollFrameScrollBar");
	CardDescrScrollbar:SetMinMaxValues(0, CardDescrScrollText:GetHeight());
	CardDescrScrollbar:SetValue(0);
	getglobal(me.name .. "_CardDescrScrollFrame"):UpdateScrollChildRect();

	LuaFunc_ShowCardImage(tonumber(strIDhex, 16));
	CommonModel_OnShow(CardBookFrameCardImage);

	getglobal(me.name .. "_CardStatus"):SetText(yaCIt.GetOwnedString(strIDhex, true));
	if strucCard.DracoMonster then
		getglobal(me.name .. "_CardReward"):SetText(yaCIt.Yellow:format(yaCIt.Locales.dracomonster));
	elseif strucCard.DracoMonsterSecret then
		getglobal(me.name .. "_CardReward"):SetText(yaCIt.Yellow:format(yaCIt.Locales.dracomonstersecret));
	elseif strucCard.EventMonster then
		getglobal(me.name .. "_CardReward"):SetText(yaCIt.Yellow:format(yaCIt.Locales.eventmonster));
    elseif strucCard.FireEvent then
        getglobal(me.name .. "_CardReward"):SetText(yaCIt.Yellow:format(yaCIt.Locales.fireevent));
    elseif strucCard.FlowerEvent then
        getglobal(me.name .. "_CardReward"):SetText(yaCIt.Yellow:format(yaCIt.Locales.flowerevent));
    elseif strucCard.MaskedBall then
        getglobal(me.name .. "_CardReward"):SetText(yaCIt.Yellow:format(yaCIt.Locales.maskedball));
	elseif strucCard.SpringRainEvent then
		getglobal(me.name .. "_CardReward"):SetText(yaCIt.Yellow:format(yaCIt.Locales.springrainevent));
	elseif strucCard.MusicEvent then
		getglobal(me.name .. "_CardReward"):SetText(yaCIt.Yellow:format(yaCIt.Locales.musicevent));
	elseif strucCard.CraftingEvent then
		getglobal(me.name .. "_CardReward"):SetText(yaCIt.Yellow:format(yaCIt.Locales.craftingevent));
	elseif strucCard.ColorWeaveEvent then
		getglobal(me.name .. "_CardReward"):SetText(yaCIt.Yellow:format(yaCIt.Locales.colorweaveevent));
	elseif strucCard.FairyTailEvent then
		getglobal(me.name .. "_CardReward"):SetText(yaCIt.Yellow:format(yaCIt.Locales.fairytailevent));
 -- elseif strucCard.AnniversaryEvent then
 --     getglobal(me.name .. "_CardReward"):SetText(yaCIt.Yellow:format(yaCIt.Locales.anniversaryevent));
	elseif strucCard.JuiceEvent then
		getglobal(me.name .. "_CardReward"):SetText(yaCIt.Yellow:format(yaCIt.Locales.juiceevent));
    elseif strucCard.PumpkinEvent then
        getglobal(me.name .. "_CardReward"):SetText(yaCIt.Yellow:format(yaCIt.Locales.pumpkinevent));
    elseif strucCard.AutumnBanquetEvent then
        getglobal(me.name .. "_CardReward"):SetText(yaCIt.Yellow:format(yaCIt.Locales.autumnbanquetevent));
    elseif strucCard.SnowflakeEvent then
        getglobal(me.name .. "_CardReward"):SetText(yaCIt.Yellow:format(yaCIt.Locales.snowflakeevent));
	elseif strucCard.EventInstanceAnniversary then
		getglobal(me.name .. "_CardReward"):SetText(yaCIt.Purple:format(yaCIt.Locales.eventinstanceanniversary));
	elseif strucCard.EventInstanceSpring then
		getglobal(me.name .. "_CardReward"):SetText(yaCIt.Purple:format(yaCIt.Locales.eventinstancespring));
 -- elseif strucCard.EventInstanceValentine then
 --     getglobal(me.name .. "_CardReward"):SetText(yaCIt.Purple:format(yaCIt.Locales.eventinstancevalentine));
	elseif strucCard.EventInstanceHalloween then
		getglobal(me.name .. "_CardReward"):SetText(yaCIt.Purple:format(yaCIt.Locales.eventinstancehalloween));
    elseif strucCard.EventInstanceChristmas then
        getglobal(me.name .. "_CardReward"):SetText(yaCIt.Purple:format(yaCIt.Locales.eventinstancechristmas));
	elseif strucCard.MonsterItem then
		getglobal(me.name .. "_CardReward"):SetText(yaCIt.Yellow:format(yaCIt.Locales.monsteritem));
	elseif strucCard.MonsterItemSpecial then
		getglobal(me.name .. "_CardReward"):SetText(yaCIt.Yellow:format(yaCIt.Locales.monsteritemspecial));
	elseif strucCard.QuestMonsterI then
		getglobal(me.name .. "_CardReward"):SetText(yaCIt.Yellow:format(yaCIt.Locales.questmonsterI));
	elseif strucCard.QuestMonsterII then
		getglobal(me.name .. "_CardReward"):SetText(yaCIt.Yellow:format(yaCIt.Locales.questmonsterII));
	elseif strucCard.QuestMonsterSpecial then
		getglobal(me.name .. "_CardReward"):SetText(yaCIt.Yellow:format(yaCIt.Locales.questmonsterspecial));
	elseif strucCard.QuestReward then
		getglobal(me.name .. "_CardReward"):SetText(yaCIt.Yellow:format(yaCIt.Locales.questreward));
	elseif strucCard.QuestRewardSecret then
		getglobal(me.name .. "_CardReward"):SetText(yaCIt.Yellow:format(yaCIt.Locales.questrewardsecret));
	elseif strucCard.NotObtainable then
		getglobal(me.name .. "_CardReward"):SetText(yaCIt.Yellow:format(yaCIt.Locales.notobtain));
	elseif strucCard.MonsterSpawn then
		getglobal(me.name .. "_CardReward"):SetText(yaCIt.Yellow:format(yaCIt.Locales.monsterspawn));
	elseif strucCard.MonsterSpawnRandom then
		getglobal(me.name .. "_CardReward"):SetText(yaCIt.Yellow:format(yaCIt.Locales.monsterspawnrandom));
	elseif strucCard.PackageCard1 then
		getglobal(me.name .. "_CardReward"):SetText(yaCIt.Yellow:format(yaCIt.Locales.packagecard1));
	elseif strucCard.GashaCard1 then
		getglobal(me.name .. "_CardReward"):SetText(yaCIt.Yellow:format(yaCIt.Locales.gashacard1));
	elseif strucCard.GashaCard2 then
		getglobal(me.name .. "_CardReward"):SetText(yaCIt.Yellow:format(yaCIt.Locales.gashacard2));
	elseif strucCard.GashaCard3 then
		getglobal(me.name .. "_CardReward"):SetText(yaCIt.Yellow:format(yaCIt.Locales.gashacard3));
	elseif strucCard.GashaCard4 then
		getglobal(me.name .. "_CardReward"):SetText(yaCIt.Yellow:format(yaCIt.Locales.gashacard4));
	elseif strucCard.PlayerNetGames then
		getglobal(me.name .. "_CardReward"):SetText(yaCIt.Purple:format(yaCIt.Locales.playernetgames));
	elseif strucCard.FutureZone then
		getglobal(me.name .. "_CardReward"):SetText(yaCIt.Yellow:format(yaCIt.Locales.futurezone));
	end
	CardBookFrameCard:Show();
	if getglobal(me.name .. "_Chk_ShowDetails"):IsChecked() then
		me.ShowSelectedCardDetails(strIDhex);
	end
end

--[[ =================================================================
  Creates the 'additional info' overlay and shows it
================================================================= --]]
function me.ShowSelectedCardDetails(strIDhex)
	local data = {};
	local strucCard = yaCIt.CardList[strIDhex];

	data.ID = 100;

	local locales = yaCIt.Locales.book.cardoverlay;
 	local label = locales.altnamelabel;
	for cName, cards in pairs(yaCIt.NameLookup) do
		for _, IDhex in ipairs(cards) do
			if IDhex == strIDhex and cName ~= strucCard.cName then
				table.insert(data, { Left = label, Right = cName } );
				label = "  ";
			end
		end
	end

    table.insert(data, { Left = locales.cardid, Right = tonumber(strIDhex, 16) .. " / " .. strIDhex } );
 -- table.insert(data, { Left = locales.monsterid, Right = "?????" } );

 -- local strRare = LuaFunc_GetString("SYS_RARECOLOR_" .. strucCard.Rarity-2) .. " (" .. (strucCard.Rarity-2) .. ")";
 -- table.insert(data, { Left = locales.rarelabel, Right = strRare } );
 -- table.insert(data, { Left = locales.resistlabel, Right = strucCard.Resist } );

	if AAH_SavedHistoryTable then
		if AAH_SavedHistoryTable[tonumber(strIDhex,16)] then
			local AAH_Item = AAH_SavedHistoryTable[tonumber(strIDhex,16)];
			if ( AAH_Item.min ~= nil and AAH_Item.avg ~= nil and AAH_Item.max ~= nil ) then
				table.insert(data, { Left = locales.aahlabel, Right = AAH_Item.min .. " / " .. AAH_Item.avg .. " / " .. AAH_Item.max } );
			end
		end
	end

	label = locales.owners;
	for CharName, Data in pairs(yaCIt_Persistent[yaCIt.Realm]) do
		if CharName ~= yaCIt.Player and Data.Owned and Data.Owned[strIDhex] == 1 then
			table.insert(data, { Left = label, Right = CharName } );
			label = " ";
		end
	end
	
	data.title = locales.title;
	me.ShowInfoOverlay(data);
end

--[[ =================================================================
  Shows the overlay frame (hides the frame if data == nil)
================================================================= --]]
function me.ShowInfoOverlay(data)
	local frame = getglobal(me.name .. "_InfoOverlay");
	frame.constructing = true;
	frame:Hide();
	if type(data) == "table" then
		frame:SetText(data.title);

		for _, line in ipairs(data) do
		frame:AddDoubleLine(line.Left, yaCIt.Yellow:format(line.Right));
		end
		frame:SetID(data.ID);
		frame:ClearAllAnchors();
		frame:SetAnchor("TOPLEFT", "BOTTOMLEFT", yaCardBook_CardIDhex, (230-frame:GetWidth())/(GetUIScale()*2), 5);

		frame.LastHeight = frame:GetHeight();
		frame.LastWidth = frame:GetWidth();
	end
	    frame.constructing = false;
end

--[[ =================================================================
  Manages anything around the listbox and status informations 
  if the content of the listbox changes
================================================================= --]]
function me.UpdateFrame()
	local strPerc = math.floor(yaCIt.NumOwned / yaCIt.NumCards * 10000) / 100 .. "%";
	local strTotal = yaCIt.Locales.book.titlestats:format(yaCIt.NumOwned, yaCIt.NumCards, strPerc);
	getglobal(me.name .. "_Title"):SetText(yaCIt.Logo .. " " .. SYSTEM_CARDBOOK .. strTotal);

	if me.FrameItemListSize >= #me.CardList_Sorted then
		me.ScrollBar_Value = 0;
		CardBookFrameScrollBar:Hide();
	else
		CardBookFrameScrollBar:Show();
		CardBookFrameScrollBar:SetMinMaxValues(0, #me.CardList_Sorted - me.FrameItemListSize);
		me.ScrollBar_Value = CardBookFrameScrollBar:GetValue();
	end
	me.ItemListSelected = 1;
	me.FillListbox();
	
	if #me.CardList_Filtered == yaCIt.NumCards then
		getglobal(me.name .. "_FilterCount"):Hide();
	else
		getglobal(me.name .. "_FilterCount"):Show();
		getglobal(me.name .. "_FilterCountText"):SetText(yaCIt.Locales.book.filtercountlabel .. me.NumOwned_Filtered .. "/" .. #me.CardList_Filtered);
	end
	me.ShowSelectedCard();
end

--[[ =================================================================
  Event function OnLoad; realigns reused object of the original
  CardBookFrame and hides the obsolete objects. New object are added
  with the XML file. Sets all language dependent labels
================================================================= --]]
function me.OnLoad(this)

	CardBookFrame:SetSize(628,540); -- Resize master frame
	CardTitleFrame:SetSize(582,28); -- resize drag area

	-- add new background; here to access the original background layer
	local CBFtexture = CreateUIComponent("Texture", me.name .. "_Texture", "CardBookFrame");
	CBFtexture:SetSize(1024, 1024);
	CBFtexture:SetTexture("Interface/AddOns/yaCIt/data/yaCardBookFrame");
    CBFtexture:ClearAllAnchors();
    CBFtexture:SetAnchor("TOPLEFT", "TOPLEFT", CardBookFrame, 0, 0);		
	CardBookFrame:SetLayers(1, CBFtexture);

	CardBookFrameText:Hide(); -- Hide old window title (already overlayed by new background)
	CardBookFrameTab1:Hide(); -- Hide very obsolete tab
	CardBookFrameType:Hide(); -- Hide race select dropdown; need new one
	CardBookFrameCountText:Hide(); -- Hide old filter counter
	CardBookFrameCardSP:Hide(); -- Hide old stat label
	CardBookFrameCardTip:Hide(); -- Hide old description box
	CardBookFrameCardNoteScrollFrame:Hide(); -- remove description scroll box; need new one

	for idx = 1, 18 do -- resize lines of the scroll list
		_G["CardBookFrameList" .. idx]:SetSize(268,16);
		_G["CardBookFrameList" .. idx .. "Text"]:SetSize(263,12);
		_G["CardBookFrameList" .. idx .. "Text"]:SetJustifyHType("LEFT");
		_G["CardBookFrameList" .. idx .. "Text"]:ClearAllAnchors();
		_G["CardBookFrameList" .. idx .. "Text"]:SetAnchor("LEFT","LEFT",_G["CardBookFrameList" .. idx],10,0);
	end

	CardBookFrameList1:ClearAllAnchors(); -- set new position of the scroll list
	CardBookFrameList1:SetAnchor("TOPLEFT","TOPLEFT",CardBookFrame,20,151);

	CardBookFrameScrollBar:ClearAllAnchors(); -- set new position of the scroll bar
	CardBookFrameScrollBar:SetAnchor("TOPLEFT","TOPRIGHT",CardBookFrameList1,0,19);
	CardBookFrameScrollBar:SetSize(16,250); -- and resize it
	
	CardBookFrameCard:ClearAllAnchors(); -- set new position of the card image area
	CardBookFrameCard:SetAnchor("TOPLEFT","TOPLEFT",CardBookFrame,320,84);

	local book = yaCIt.Locales.book;
	getglobal(me.name .. "_FilterBoxLabel"):SetText(book.filterboxlabel);
	getglobal(me.name .. "_Chk_Invert_Text"):SetText(book.chkinvertlabel);
	getglobal(me.name .. "_Chk_ShowDetails_Text"):SetText(book.chkshowdetailslabel);

	CardBookFrame:UnregisterEvent("CARDBOOKFRAME_UPDATE");
end

--[[ =================================================================
  Event function OnShow
  --> overloaded function <--
================================================================= --]]
--function me.OnShow(this)
function OnShow_CardFrame(this)
	if me.DD_Zone_Value == -3 then
		local zoneID = yaCIt.GetZoneID();
		local zoneName = GetZoneLocalName(zoneID);
		if zoneID == 400 then
			zoneName = TEXT("ZONE_DEFAULT");
		elseif not zoneName then
			zoneName = string.format("<%s>", TEXT("GUILD_TEXT_ZONE_UNKOWN"));
		end
		getglobal(me.name .. "_DD_ZoneText"):SetText(zoneName);
	end

	getglobal(me.name .. "_InfoOverlay"):SetID(0); -- fallback if stat sums were shown
	
	me.CreateFilteredList();
	me.UpdateFrame();
end

--[[ =================================================================
  Event function OnEvent
  Additional matches the zone DD label on LOADING_END event if 
  'current zone' is selected
  --> hooked function <--
================================================================= --]]
me.OnEvent = CardFrame_OnEvent;
function CardFrame_OnEvent(this, event)
	if event ~= "CARDBOOKFRAME_UPDATE" then
		me.OnEvent(this, event);
	end
end

_G[me.name] = me;
